# User Manual

## Required Information to query XNAT

* The **address** of your XNAT-Server
* Your **Username** and **Password**
* A **Root-Type**
* A **Query** which asks for some fields of the chosen Root-Type
  * the following definitions are adapted from the [pyxnat-Dokumentation](https://pythonhosted.org/pyxnat/features/search_engine.html):
  * A ** *query* ** is an unordered list that contains
    * 1 or more ** *constraint(s)* **
    * 0 or more ** *sub-queries* ** (lists as this one)
    * 1 ** *comparison method* ** between the constraints (`AND` or `OR`)
  * A ** *constraint* ** is an ordered tuple that contains
    * 1 valid ** *searchable_type/searchable_field* **
    * 1 ** *operator among* ** `=, <, >, <=, >=, LIKE`
  * the GUI will help you to build your query
* A list of **Fields** you want to get back from the server

## How to work with the GUI

### <a name="overview"></a>GUI Overview

![Image][gui_overview]

 *Elements for Query-Creating*
1. Buttons for saving/loading Queries
2. Buttons for Server Settings and sending Query
3. Dropdownbox for choosing the Root-Type
4. Tree for creating the query-constraints
5. List of all defined Fields of chosen root-type, checked fields will be requested and shown in result-table

 *Elements for Downloading*
6. Buttons for Result-Table (Toggle Selection and Download selected Items (enabled if download is not defined for this Root-Type (see Root-Types: Structure of Root-Types)))
7. Result-Table

 *Elements of Diagram*
8. Dropdownbox for choosing type which will be shown in diagram
9. List for creating Substrings, which will be shown in diagram
10. Diagram

 *Other*
11. Statusbar, shows result-count (when query was sent)

### Creating a Query

1. Choose a Root-Type using the Dropdownbox (see [GUI Overview](#overview): 3)
  * Root-Types are defined as JSON-files, located at `../datatypes`
2. Create the query-constraints using the tree (see [GUI Overview](#overview): 4)
  * click on `...`
  * choose a logical connective (AND/OR) or a type (depends on the chosen Root-Type)
  * you can choose a single type at the beginning, to create a very simple query
  * you can nest operations and types
  * if you choose a type, make sure you choose an operator and a value too
  * **use % for wildcards**
  * **delete a line (and its children) by using right-click on this element**
3. Choose some fields for your query using the "Result-Field"-list (see [GUI Overview](#overview): 5)
  * there are some fields you can't deselect (These are important for downloading files for this Root-Type)
  * if you don't choose any field, the server will give you all fields of this type
  * choose every field, you want to use in the diagram

Example Query:

![Image][query_example]

### Sending a Query

1. Click on the button "Server Settings" (see [GUI Overview](#overview): 2)
  * type your username, password and host-address ( **including protocol** , like http, https)
  * you can start the gui with this settings by calling it like this:
  ``python xsa_gui.py --user USERNAME:PASSWORD --host http://HOST``
2. Click on the button "Send Query" (see [GUI Overview](#overview): 2)
  * if no error occurs, the results will be shown in the Result-Table (see [GUI Overview](#overview): 7) and the count of your Results will be shown at the statusbar (see [GUI Overview](#overview): 11)

### Downloading Files

1. Select some results at the Result-Table (see [GUI Overview](#overview): 7)
  * Maybe use the "Toggle-Selection"-Button (see [GUI Overview](#overview): 6)
2. Click the "Download Selected Items"-Button (see [GUI Overview](#overview): 6)
  * A window for choosing a destination-folder will appear
  * A spinner will appear at the single elements in the Result-Table, at the beginning of the download and disappears when the download is finished

### Using the Diagram

1. Choose one of your requested fields by using the Dropdownbox (see [GUI Overview](#overview): 8)
2. Add some substrings to the "Chart-Value"-List (see [GUI Overview](#overview): 9) by clicking on `...`
  * **delete substrings by rightclicking on them**
  * its not case-sensitive
  * the diagram will update immediately

![Image][diagram_example]

## Root-Types
### Introduction

In this program you have to define root-types for using them in queries. These Root-Types are saved as JSON-Files at `../datatypes`. If the files aren't correct, they will simply not shown in the Root-Type-Dropdownbox. The names of these files are not important, but you have to create a correct JSON-Object, including the fields as defined in the next chapter. define only one type in a single file.

You can use the XNATRestClient to get the 	necessary information about the Root-Type you want to define. You can download this software [here](https://wiki.xnat.org/display/XNAT/XNATRestClient).
How to use this program is described in the following chapter.

### Structure of Root-Types

One Root-Type is defined by three fields:
####"root-type"
  * Type: string
  * Description: name of the root-type you want to define for your search
  * Request types for your host by using the XNATRestClient: `XNATRestClient -host http://yourHOST -u yourUSER -p yourPASSWORD -remote "/data/search/elements?format=json" -m GET`

#### "REST-API" (not required)
  * Type: string
  * Description: REST-Interface (see the [related Documentation](https://wiki.xnat.org/display/XNAT16/Downloading+Files+via+XNAT+REST+API))
  * Write keys (see ["fields"](#fields)) of required fields in `{}`
  * If this field is not specified, the Download-button will be enabled for this Root-Type

#### <a name="fields"></a>"fields"
  * Type: list
  * Description: a list of fields for this type
  * Request the fields of a type (for example xnat:mrSessionData) by using the XNATRestClient: `XNATRestClient -host http://yourHOST -u yourUSER -p yourPASSWORD -remote "/data/search/elements/xnat:mrSessionData?format=json" -m GET`
  * Every field-definition must contain this properties:
    * `"label"`: string, just for the GUI (you can pick what you want)
    * `"field"`: string, you get it from the XNATRestClient. It has to have this form: `element_name` + `/` + `i` (`element_name` is the name given by the XNATRestClient, it's actual the `root-type`
    * `"required"`: `true`/`false`, if `true`: this field can't be deselected, set `true`, if this field is necessary for downloading via the defined REST-Interface
    * `"key"`: string, mainly to show the label at the result table and the diagram-dropdownbox, if not set, the `"key"` will be shown in the result-table and the diagram-dropdownbox, mostly the key is just the `"field"`-name in lower case and without `root-type/`

Example:

```json
{
"root-type":"xnat:mrScanData",
"REST-API":"/data/archive/projects/{xnat_mrsessiondata_project}/subjects/{xnat_mrsessiondata_subject_id}/experiments/{xnat_mrsessiondata_session_id}/scans/{id}/resources/DICOM/files?format=zip",
"fields":[
{"label":"Type", "field":"xnat:mrScanData/TYPE", "required":true, "key": "type"},
{"label":"Project-ID", "field":"xnat:mrSessionData/PROJECT", "required":true, "key":"xnat_mrsessiondata_project"}
]
}
```
If you neet more examples, there are some functioning Root-Types at the `../datatypes` folder (*xnat:mrScanData*, *xnat:mrSessionData*, *xnat:projectData*, *xnat:subjectData*).


### Trouble-Shooting

#### My new Root-Type is not available in the Root-Type-Dropdownbox!

There is a mistake in your Root-Type-File.
Common Errors are:
* You ended the `"fields"`-list with a comma: ,
* You forgot a comma somewhere

If you can't figure out whats the problem, you can use an editor with json-code highlighting (like atom or gedit). Some Syntax -Errors can be shown immediately.


[gui_overview] : https://mosel.cbs.mpg.de/xnat/xnat-scan-ambassador/raw/master/READMEimages/GUI_overview.png
[query_example] : https://mosel.cbs.mpg.de/xnat/xnat-scan-ambassador/raw/master/READMEimages/query_example.png
[diagram_example] : https://mosel.cbs.mpg.de/xnat/xnat-scan-ambassador/raw/master/READMEimages/diagram_example.png
